package Shell::SSH;
use strict;
use warnings;
use base qw( Shell );
our($VERSION, $AUTOLOAD);

$VERSION = '0.01';

# this is just needed for checking $VERSION of Shell.pm which we inherit from
# alternative: don't "use warnings;" else this modules throws "Subroutine AUTOLOAD redefined at Shell/SSH.pm line ..."
# we try to imitate "use base qw( Shell 0.4 );" which of course will never work since it "arrays" all values inside qw()
# but it also calls 'require' under the hood - rendering a "use base ('Shell 0.4');" useless, too, hmmm.....
# question: can this already be done in a more formal way?
BEGIN { my $requires = 0.4; die "Shell version $requires required--this is only version $Shell::VERSION" if ($Shell::VERSION lt $requires); }

use Data::Dumper;
use IPC::Run qw( start pump finish timeout );

# debugging for IPC::Run
# $ENV{IPCRUNDEBUG} = 2;

local *IN;
local *OUT;
local *ERR;

$Shell::hasHandler = 1;

sub new {
  my $class = shift;
  my $self = { @_ };
  bless $self, $class;
  $self->_init();
  $self->_start();
  return $self;
}

sub DESTROY { }

sub AUTOLOAD {
  my $self = shift if ref $_[0] && $_[0]->isa( 'Shell' );
  my $cmd = $AUTOLOAD;
  $cmd =~ s/^.*:://;
  #print Dumper(@_);
  #exit;
  if ($self) {
    # TODO: handle asynchronizity(?) here!
    return $self->_run_command($cmd, @_);
  }
}

sub _init {
  my $self = shift;
  $self->{method} ||= 'ssh';
}


sub _start {
  my $self = shift;

  my @cmd = ();
  die "please supply method" if (!$self->{method});
  die "please supply target" if (!$self->{target});
  
  push @cmd, $self->{method};
  push @cmd, @{$self->{args}} if ($self->{args});
  push @cmd, $self->{target} if ($self->{target});

  my $cmd = join(" ", @cmd);
  #print "command: ", $cmd, "\n";

  $self->{handle} = start
     \@cmd,
        '<pipe', \*IN,
        '>pipe', \*OUT,
        '2>pipe', \*ERR,
        timeout( 5 )
     or die "could not open IPC::Run - handle: $?" ;
  
}

sub _stop {
  my $self = shift;
  finish $self->{handle};
}

sub _run_command {
  my $self = shift;
  my $rcommand = shift;
  my $rargs = join(' ', @_);
  $rargs ||= '';
  my $rcommandstring = join(' ', $rcommand, $rargs);
  #open IN, '';
  print IN $rcommandstring;
  close IN;
  $self->_read_output();
  return '';
}

sub _read_output {
  my $self = shift;
  $self->{buffer} = [];
  my $buffer = '';
  #print "=read\n";
  #print Dumper($self->{handle});
  while (<OUT>) {
    #print "line: ", $_, "\n";
    chomp();
    push @{$self->{buffer}}, $_;
    $buffer .= $_ . "\n";
  }
  #print "=read okay\n";
  # TODO: async!
  #return $buffer;
  $self->{response}->($buffer);
}

sub get_output {
  my $self = shift;
  return $self->{buffer};
}

sub disconnect {
  my $self = shift;
  $self->_stop();
}

1;
