package Shell::SSH;
use strict;
use warnings;
use base qw( Shell );
our($VERSION, $AUTOLOAD);

$VERSION = '0.01';

use Data::Dumper;
use IPC::Run qw( start pump finish timeout );

# debugging
# $ENV{IPCRUNDEBUG} = 2;

# using corkscrew to get through certain http proxies
my $corkscrew = '/usr/local/bin/corkscrew';
# good tunnels might be:
my $tunnels = [
  'proxy.nida.ac.th:8080',   # Thailand
  '210.0.200.211:80',         # Hong Kong    (ssh1 only)
  '200.27.152.3:80',          # Chile
];

local *IN;
local *OUT;
local *ERR;

$Shell::hasHandler = 1;

sub new {
  print "new\n";
  my $class = shift;
  my $self = { @_ };
  bless $self, $class;
  $self->_init();
  $self->_start();
  return $self;
}

sub DESTROY { }

sub AUTOLOAD {
  my $self = shift;
  print "==================AUTO\n";
  exit;
    shift if ref $_[0] && $_[0]->isa( 'Shell' );
    #my $cmd = $AUTOLOAD;
    my $cmd = shift;
    print "cmd: $cmd\n";
    $cmd =~ s/^.*:://;
    print "===========COMMAND: $cmd", "\n";
}

sub _init {
  print "_init\n";
  my $self = shift;

    #$self->{isHandler} = 1;

    $self->{options}{proxy_host} = $tunnels->[0]
      if $self->{options}{proxy_enabled} && !$self->{options}{proxy_host};
  
    $self->{method} ||= 'ssh';
  
    if ($self->{options}{proxy_enabled} && $self->{options}{proxy_host}) {
      my @proxy = split(/:/, $self->{options}{proxy_host});
      $self->{proxy}{host} = $proxy[0];
      if ($proxy[1]) {
        $self->{proxy}{port} = $proxy[1];
      }
      $self->{proxy}{port} ||= 80;
    }

}


sub _start {
  print "_start\n";
  my $self = shift;

  my @cmd = ();
  die "please supply method" if (!$self->{method});
  die "please supply target" if (!$self->{target});
  
  if ($self->{proxy}) {
    push @cmd, '-o', "ProxyCommand $corkscrew $self->{proxy}{host} $self->{proxy}{port} " . '%h %p';
  }
  
  push @cmd, $self->{method};
  push @cmd, @{$self->{args}} if ($self->{args});
  push @cmd, $self->{target} if ($self->{target});

  my $cmd = join(" ", @cmd);
  #print "command: ", $cmd, "\n";

  $self->{handle} = start
     \@cmd,
        '<pipe', \*IN,
        '>pipe', \*OUT,
        '2>pipe', \*ERR,
        timeout( 3 )
     or die "could not open IPC::Run - handle: $?" ;
  
}

sub _stop {
  my $self = shift;
  finish $self->{handle};
}

sub _read_output {
  my $self = shift;
  $self->{buffer} = [];
  while (<OUT>) {
    chomp();
    push @{$self->{buffer}}, $_;
  }
}

sub run {
  my $self = shift;
  my $rcommand = shift;
  print IN $rcommand;
  close IN;
  $self->_read_output();
}

sub get_output {
  my $self = shift;
  return $self->{buffer};
}

sub disconnect {
  my $self = shift;
  $self->_stop();
}

1;
